unit unEnvelope;

interface
uses classes;
type
TEnvaddToLog = procedure(loglevel:integer;msg:string);

TEnvelope = class(TObject)
    private
    fmessagetype : string;
    procedure setmessagetype(val:string);
    public
    messagedata  : string;
    owner : string;      //username
    id : string;        //unique message id
    callbackid : string; //used for delivery reports to identify the original msg
    status : string;     //idle,sending,transmitted,delivered,received
    recepient : string;
    subject : string;    //The length is less then 1024
    history : string;
    retrycount : integer;
    operator : string;
    flag     : string; //'D' = del

    //message aging
    creationdate : int64;

    originator : string;

    property messagetype : string read fmessagetype write setmessagetype;

    //file management
    procedure reset;

    function toXml:string;
    procedure fromXml(xml:string;data:boolean=true);
    function makeID : string;
    constructor create(inOwner:string);
    destructor  destroy; override;

end;                                                

function chopfront(s:string;num:integer):string;
function chopend(s:string;num:integer):string;

implementation
uses sysutils;

procedure TEnvelope.setMessageType(val:string);
Begin
    if val='' then fmessageType := 'GSMSMS' else fmessageType := val;
End;

procedure TEnvelope.reset;
Begin
   id := inttostr(random(9999999));
   status := 'idle';
   creationdate := 0;
   callbackid := id;
   retrycount := 6;
   messagetype := '';
   messagedata := '';
End;

destructor  TEnvelope.destroy;
Begin
  inherited destroy;
End;

function toXmlReady(const plain:string):string;
var
    x,cn : integer;
    c : byte;
    ts : string;
Begin
ts := '';
cn := length(plain);
for x := 1 to cn do begin
   c:=ord(plain[x]);
   if c = 38 then ts := ts+'&amp;' else
   if c = 62 then ts := ts+'&gt;' else
   if c = 60 then ts := ts+'&lt;' else
   if c = 39 then ts := ts+'&apos;' else
   if c = 34 then ts := ts+'&quot;' else
   ts := ts+chr(c);
end;
Result := ts;
End;

function chopfront(s:string;num:integer):string;
Begin
   if (length(s)>=num) then Begin
      Result := copy(s,num+1,length(s)-num);
   End else begin
      Result := '';
   end;
End;

function chopend(s:string;num:integer):string;
Begin
   Result := copy(s,1,length(s)-num);
End;

function getXMLValue(tagname,data:string):string;
var p1 : integer;
    ret : string;
Begin
    p1 := pos('<'+tagname+'>',data);
    ret := chopfront(data,p1+length(tagname)+1);
    p1 := pos('</'+tagname+'>',ret);
    ret := chopend(ret,length(ret)-p1+1);
    Result := ret;
End;

function TEnvelope.toXml:string;
var ret:string;
Begin                                  
  ret := '<Envelope>'+
             '<Filename></Filename>'+
             '<MessageType>'+messagetype+'</MessageType>'+
             '<Owner>'+toXmlReady(owner)+'</Owner>'+
             '<Id>'+id+'</Id>'+
             '<CallbackId>'+toXmlReady(callbackid)+'</CallbackId>'+
             '<CreationDate>'+inttostr(creationDate)+'</CreationDate>'+
             '<Originator>'+toXmlReady(originator)+'</Originator>'+
             '<Recepient>'+toXmlReady(recepient)+'</Recepient>'+
             '<Subject>'+toXmlReady(subject)+'</Subject>'+
             '<RetryCount>'+inttostr(retrycount)+'</RetryCount>'+
             '<History>'+toXmlReady(history)+'</History>'+
             '<Status>'+Status+'</Status>'+
             '<Operator>'+toXmlReady(Operator)+'</Operator>';
  ret :=ret+'<Message>'+toXmlReady(messageData)+'</Message>';
  ret := ret+'</Envelope>';        
   Result := ret;
End;

procedure TEnvelope.fromXml(xml:string;data:boolean=true);
var
   creatdat : string;
Begin
   messagetype := getXMLValue('MessageType',xml);
   owner := getXMLValue('Owner',xml);
   id := getXMLValue('Id',xml);
   callbackId := getXMLValue('CallbackId',xml);
   originator := getXMLValue('Originator',xml);
   recepient := getXMLValue('Recepient',xml);
   subject := getXMLValue('Subject',xml);
   messageData := getXMLValue('Message',xml);
   retrycount := strtoint(getXMLValue('RetryCount',xml));
   history := getXMLValue('History',xml);
   status := getXMLValue('Status',xml);
   operator := getXMLValue('Operator',xml);
End;


constructor TEnvelope.create(inowner : string);
Begin
   id := inttostr(random(9999999));
   status := 'idle';
   messagetype := 'SMS:TEXT';
   callbackid := id;
   owner := inowner;
   retrycount := 6;
End;

function TEnvelope.makeid : string;
Begin
    id := inttostr(random(9999999));
    Result := id;
End;

initialization
randomize;

end.
